#include "peakpick.h"
#include "biquad.h"

/* parms: peak picking parameters, default values in brackets
 * sparms(thresh,alpha,cutoff,window,plotf,threshfn,picker)
 *
 *	   [<----post----|--pre-->]
 *	.................|.............
 *	time->           ^now
 */
struct _pickparams_t {
	/** thresh: offset threshold [0.033 or 0.01] */
	float threshold; 	
	float maxthreshold ;
  /** alpha: normalisation exponent [9] */
	float alpha; 			
	/** cutoff: low-pass filter cutoff [0.34, 1] */
	float cutoff; 		
	float gain; 		
	/** tolerance [0.05] */
	float tol; 				
	/** time precision [256/22050  winlength/samplerate, fs/buffer_size */
	float tau; 				
  /** win_post: median filter window length (causal part) [8] */
	int 	win_post; 			
  /** pre: median filter window (anti-causal part) [post-1] */
	int 	win_pre; 				
  /**	threshfn: name or handle of fn for computing adaptive threshold [@median]  */
	aubio_thresholdfn_t thresholdfn;
 	/**	picker:   name or handle of fn for picking event times [@peakpick] */
	aubio_pickerfn_t pickerfn;

	fvec_t * medianarr;
	fvec_t * onset_keep;
	fvec_t * onset_peek;
	fvec_t * onset_proc;

	//fvec_t * filtarr;
	biquad_t * biquad;
};


/* returns 1 if X1 is a peak and positive */
//void peakpick(fvec_t * DF) {
int peakpick(fvec_t * DF, int pos) {
	unsigned int i, tmp=0;
	for (i=0;i<DF->channels;i++)
		tmp = (
					DF->data[i][pos] > DF->data[i][pos-1]
			&&  DF->data[i][pos] > DF->data[i][pos+1]
			&&	DF->data[i][pos] > 0.
					);
	return tmp;
}

/* T=quadpick(X): return indices of elements of X which are peaks and positive
 * exact peak positions are retrieved by quadratic interpolation
 */
/*
void quadpick(fvec_t * X){
	unsigned int i,j;
	for (i=0;i<X->channels;i++)
		for (j=0;j<X->size;j++)
		{
	I=2:(length(X)-1);
	T.bin=find((X(I)>X(I-1)) & (X(I)>X(I+1)) & (X(I)>0))+1;
	for k=1:length(T.bin), T.ebin(k)=quadint(X,T.bin(k)); end;
		}
}
*/

/** finds exact peak position by quadratic interpolation*/
	/*
float quadint(fvec_t * x,int pos) {
	if (pos<length(x)-1) {
		span=2;
		input={pos-span, pos, pos+span};
		f=input-pos;
		p=polyfit(f,x(input),2);
		a=p(1);b=p(2);c=p(3);
		f2=[f(1):0.01:f(3)];
		x2=a.*f2.^2+b.*f2+c;
		offset=f2(find(eq(x2,max(x2))));
		exactpos=input(2)+offset;
	}
	else
		exactpos=pos;
	return exactpos;
	return 0;
}
*/

float local_energy(fvec_t * f){
	float locE = 0.;
	unsigned int i,j;
	for (i=0;i<f->channels;i++)
		for (j=0;j<f->length;j++)
			locE+=SQR(f->data[i][j]);
	return locE;
}
	
float local_hfc(fvec_t * f){
	float locE = 0.;
	unsigned int i,j;
	for (i=0;i<f->channels;i++)
		for (j=0;j<f->length;j++)
			locE+=(i+1)*f->data[i][j];
	return locE;
}

/*
void spectral_pp(pitchdetector_t * p, fvec_t * mag, fvect_t * spectral_peaks) {
	float oldmag;
	unsigned int i,j;
	float min = vec_min(mag);
	float alphan = alpha_norm(mag,p->alpha);
	for (i=0;i<DF->channels;i++)
		for (j=0;j<f->length;j++)
		{
*/
			/* freq=(0:length(mag)-1).*(p.tau); not used */
/*
			mag->data[i][j]-=min;
			mag->data[i][j]/=alphan;
			{
				mag=mag-p->threshfn(mag,p_win_pre,p->win_post)';
			}
			mag-=p->thresh;
			spectral_peaks=p->picker(mag);
			spectral_peaks.freq=p.tau*(spectral_peaks.bin-1);
			spectral_peaks.mag=oldmag(spectral_peaks.bin);
		}
}
*/

/** return alpha-norm norm.
 * alpha=2 means normalise variance. 
 * alpha=1 means normalise abs value. 
 * as alpha goes large, tends to normalisation 
 * by max value.
 *
 * BUG: should not use powf :(
 */
float alpha_norm(fvec_t * DF, float alpha) {
	float tmp = 0.;
	unsigned int i,j;
	for (i=0;i<DF->channels;i++)
		for (j=0;j<DF->length;j++)
			tmp += powf(ABS(DF->data[i][j]),alpha);
	return powf(tmp/DF->length,1./alpha);
}


/** y=fn_thresh(fn,x,post,pre)
 * compute adaptive threshold at each time 
 *    fn : a function name or pointer, eg 'median', or @median
 *    x:   signal vector
 *    post: window length, causal part
 *    pre: window length, anti-causal part
 * Returns:
 *    y:   signal the same length as x
 *
 * Formerly median_thresh, used compute median over a 
 * window of post+pre+1 samples, but now works with any
 * function that takes a vector or matrix and returns a
 * 'representative' value for each column, eg
 *    medians=fn_thresh(median,x,8,8)  
 *    minima=fn_thresh(min,x,8,8)  
 * see SPARMS for explanation of post and pre
 */
/*
fvec_t * fn_thresh(aubio_thresholdfn_t * fn,fvec_t * x,int post,int pre, int pos)
	int N = x->length;
	for (i=0; i<((post>N)?post:N); i++){
		k=(i+pre>N)?i+pre:N;
		//
		m[i]=fn(x(1:k));
	}
	if (N>post+pre)
		//
		m=[m fn(buffer(x,post+pre+1,post+pre,'nodelay'))];
	for (i=N+pre-1;i>N-1;i--) {
		j=(i-post<1)?i-post:1;
		//
		m[i]=fn(x(j:end));
	}
	return m;
}
*/


/* modified version of samer pp, based on mean  rather than alpha norm and min
 */
int pimrt(fvec_t * onset,  pickparams_t * p) {
	fvec_t * onset_keep = (fvec_t *)p->onset_keep;
	fvec_t * onset_proc = (fvec_t *)p->onset_proc;
	fvec_t * onset_peek = (fvec_t *)p->onset_peek;
	float mean = 0., median = 0.;
	int length = p->win_post + p->win_pre + 1;
	unsigned int i = 0, j;


	// store onset in onset_keep
	// shift all elements but last, then write last
	/*
	for (i=0;i<channels;i++) 
	{ 
	*/
		for (j=0;j<length-1;j++) 
		{
			onset_keep->data[i][j] = onset_keep->data[i][j+1];
			onset_proc->data[i][j] = onset_keep->data[i][j];
		}
		onset_keep->data[i][length-1] = onset->data[i][0];
		onset_proc->data[i][length-1] = onset->data[i][0];
	/*
	}
	*/

	// BUG: wrong filtering, should be build with butter
	// should calculate only once too
	if (p->cutoff<1) {
		aubio_lowpassresoiir(onset_proc,p->cutoff,p->gain);
	}
	// calculate mean and median for onset_keep
	/*
	for (i=0;i<onset_keep->channels;i++) 
	{
	*/
		mean = vec_mean(onset_proc);
		for (j = 0; j < onset_proc->length; j++) 
		{
			p->medianarr->data[i][j] = onset_proc->data[i][j];
		}
		median = quick_select(p->medianarr->data[i],p->medianarr->length);
	/*
	}
	*/

	/*
	for (i=0;i<DF->channels;i++)
	{ 
	*/
		for (j=0;j<3-1;j++) 
		{
			onset_peek->data[i][j] = onset_peek->data[i][j+1];
		}
		onset_peek->data[i][2] = onset_proc->data[i][p->win_post] - median - mean * p->threshold;
	/*
	}
	*/
	//AUBIO_DBG("%f\n", onset_peek->data[0][2]);

	return (p->pickerfn)(onset_peek,1);
}




pickparams_t * new_aubio_peakpicker(void) {
	pickparams_t * t = AUBIO_NEW(pickparams_t);
	//t->maxthreshold = 2000.;
	//t->threshold = 0.33; 	
	//t->threshold = 0.082; 	
	//t->threshold = 0.033; 	
	t->threshold = 0.0668;
	/** alpha: normalisation exponent [9] */
	t->alpha=7; /* not used anymore */		
	/** cutoff: low-pass filter cutoff [0.34, 1] */
	t->cutoff=0.34;
	t->gain=1.;
	/** tolerance [0.05] */
	t->tol=0.05; /* not used here */
	/** time precision [256/22050  winlength/samplerate, fs/buffer_size */
	t->tau = 256/22050; /* not used here */				
	/** win_post: median filter window length (causal part) [8] */
	/** BUG: above 8, goes out of bounds **/
	t->win_post = 5;
	/** pre: median filter window (anti-causal part) [post-1] */
	t->win_pre = 1; 				
	/**	threshfn: name or handle of fn for computing adaptive threshold [@median]  */
	//t->thresholdfn = (aubio_thresholdfn_t)(vec_mean);
	t->thresholdfn = (aubio_thresholdfn_t)(vec_median);
	/**	picker:   name or handle of fn for picking event times [@peakpick] */
	t->pickerfn = (aubio_pickerfn_t)(peakpick);

	t->medianarr = new_fvec(t->win_post+t->win_pre+1,1);
	t->onset_keep = new_fvec(t->win_post+t->win_pre+1,1);
	t->onset_proc = new_fvec(t->win_post+t->win_pre+1,1);
	t->onset_peek = new_fvec(3,1);

	t->biquad = new_biquad(0.1600,0.3200,0.1600, -0.5949, 0.2348);

	return t;

}




/** samerpp: Peaking picking //and evaluation
 * samerpp(DF,targets,parms) //removed targets, no more eval
 *    DF: detection function (sampling rate assumed 22050/256 Hz)
 * //   targets: target onset times (in seconds)
 *    parms: peak-picking parameters (see SPARMS)
 */
float samerpp(fvec_t * DF, pickparams_t * p) {
	/* as found by JPB 28/10/2003 */
	/* remove DC component */
	//float alphanorm = 0;
	//float min = vec_min(DF);
	float athres = 0.;
	//float tmpmax = vec_max(DF);
	unsigned int i, j;

	/*
	p->maxthreshold = (p->maxthreshold  > tmpmax ) ? p->maxthreshold : tmpmax;
	p->maxthreshold = (p->maxthreshold  > 2.*tmpmax ) ? tmpmax : p->maxthreshold;
	*/
	/*
	if (alphanorm == 0.) {
		alphanorm = 0.001;
	}
	*/

	/*
	for (i=0;i<DF->channels;i++) {
		for (j = 0; j < DF->length; j++) {
			//brute force replacement for alpha norm
			DF->data[i][j] /= 1500.;
			DF->data[i][j] -= min/1500.;
		}
	}
	*/
	/* alpha norm skipped 
	alphanorm = alpha_norm(DF,p->alpha);
	for (i=0;i<DF->channels;i++) {
		for (j = 0; j < DF->length; j++) {
			DF->data[i][j] /= alphanorm;
		}
	}
	*/
	/* low pass filtering */
	
	if (p->cutoff<1) {
		// BUG: wrong filtering, should be build with butter
		aubio_lowpassresoiir(DF,p->cutoff,p->gain);
		
		//biquad_do(p->biquad, DF); //filter(b,a,x);
		/* reverse the serie */
		/*
		for (j = 0; j < DF->length; j++)
			p->medianarr->data[i][j] = DF->data[i][DF->length-j-1];
		for (j = 0; j < DF->length; j++)
			DF->data[i][j] = p->medianarr->data[i][j];
		biquad_do(p->biquad, DF);
		*/
		/* reverse back */
		/*
		for (j = 0; j < DF->length; j++)
			p->medianarr->data[i][j] = DF->data[i][DF->length-j-1];
		for (j = 0; j < DF->length; j++)
			DF->data[i][j] = p->medianarr->data[i][j];
		*/
		//now filtfilt(b,a,x)
	}
	// compute adaptive threshold 
	//DF=DF-p->thresh*alpha_norm(DF,p->alpha);
	//DF=DF-p->thresh*cauchy_norm(DF,0.3,1000);
	for (i=0;i<DF->channels;i++) {
		for (j = 0; j < DF->length; j++) {
			p->medianarr->data[i][j] = DF->data[i][j];
		}
		//athres = (p->thresholdfn)(p->medianarr);
		//calculate median on the copy
		athres = quick_select(p->medianarr->data[i],p->medianarr->length);
		for (j = 0; j < DF->length; j++) {
			// bug, should be median
				//fn_threshfn((p->threshfn),DF,p->post,p->pre);
			DF->data[i][j] -= athres;
			DF->data[i][j] -= p->threshold;// * p->maxthreshold;
		}
	}
	AUBIO_DBG("%f\n", DF->data[0][p->win_post-1]);
	//AUBIO_DBG("%f\n", athres);
	/* don't trust this filter_ioi function */
	//T=tau*filter_ioi(thresh_peaks(DF,thr+thresh)-1,DF,2);
	return (p->pickerfn)(DF, p->win_post-1);
}

