/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 */


#ifndef _SAMPLE_H
#define _SAMPLE_H

/** @file
 * Sample buffers
 *
 * Create fvec_t and cvec_t buffers 
 */

/**
 * Sample buffer type
 * 	 
 *   contains length and channels number
 */
typedef struct _fvec_t fvec_t;
/** 
 * Spectrum buffer type
 *
 *   contains length and channels number
 */ 
typedef struct _cvec_t cvec_t;

/**
 * Buffer for audio samples
 */
struct _fvec_t {
	int length;
	int channels;
	float **data;
};

/**
 * Buffer for spectral data
 */
struct _cvec_t {
	int length;
	int channels;
//	fft_data_t **data;
  float **norm;
	float **phas;
};


/* buffer function */
extern fvec_t * new_fvec(int length, int channels);
extern void del_fvec(fvec_t *s);
float fvec_read_sample(fvec_t *s, int channel, int position);
void  fvec_write_sample(fvec_t *s, float data, int channel, int position);
float * fvec_get_channel(fvec_t *s, int channel);
void fvec_put_channel(fvec_t *s, float * data, int channel);
extern cvec_t * new_cvec(int length, int channels);
extern void del_cvec(cvec_t *s);

#endif /* _SAMPLE_H */
