/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

/* default values : alfa=4, beta=3, threshold=0.25 */

#include "aubio_priv.h"
#include "mathutils.h"
#include "tss.h"

struct _aubio_tss_t 
{
	float thrs;
	float alfa;
	float beta;
	float parm;
	fvec_t *theta1;
	fvec_t *theta2;
	fvec_t *oft1;
	fvec_t *oft2;
	fvec_t *dev;
};

aubio_tss_t * new_aubio_tss(float thrs, float alfa, float beta, int size, int overlap,int channels)
{
	aubio_tss_t * o = AUBIO_NEW(aubio_tss_t);
	o->thrs = thrs;
	o->alfa = alfa;	
	o->beta = beta;	
	o->parm = thrs*TWO_PI*overlap/size;
	o->theta1 = new_fvec(size,channels);
	o->theta2 = new_fvec(size,channels);
	o->oft1 = new_fvec(size,channels);
	o->oft2 = new_fvec(size,channels);
	o->dev = new_fvec(size,channels);
	return o;
}

void del_aubio_tss(aubio_tss_t *s)
{
	free(s->theta1);
	free(s->theta2);
	free(s->oft1);
	free(s->oft2);
	free(s->dev);
	free(s);
}

void aubio_tss_do(aubio_tss_t *o, cvec_t * input, cvec_t * trans, cvec_t * stead)
{
	unsigned int i,j,test;	
	unsigned int nbins = input->length;
	for (i=0;i<input->channels; i++)	{
		for (j=0;j<nbins; j++)	{
			o->dev->data[i][j] = unwrap2pi(
					input->phas[i][j]
					-2.0*o->theta1->data[i][j]+
					o->theta2->data[i][j]);

			/* transient analysis */
			test = (ABS(o->dev->data[i][j]) > o->parm*o->oft1->data[i][j]);
			trans->norm[i][j] = input->norm[i][j] * test;
			trans->phas[i][j] = input->phas[i][j] * test;

			/* steady state analysis */
			test = (ABS(o->dev->data[i][j]) < o->parm*o->oft2->data[i][j]);
			stead->norm[i][j] = input->norm[i][j] * test;
			stead->phas[i][j] = input->phas[i][j] * test;

			/*increase sstate probability for sines */
			test = (trans->norm[i][j]==0.);
			o->oft1->data[i][j]  = test;
			test = (stead->norm[i][j]==0.);
			o->oft2->data[i][j]  = test;
			test = (trans->norm[i][j]>0.);
			o->oft1->data[i][j] += o->alfa*test;
			test = (stead->norm[i][j]>0.);
			o->oft2->data[i][j] += o->alfa*test;
			test = (o->oft1->data[i][j]>1. && trans->norm[i][j]>0.);
			o->oft1->data[i][j] += o->beta*test;
			test = (o->oft2->data[i][j]>1. && stead->norm[i][j]>0.);
			o->oft2->data[i][j] += o->beta*test;

			/* swap old phase data (need to remember 2 frames behind)*/
			o->theta2->data[i][j] = o->theta1->data[i][j];
			o->theta1->data[i][j] = input->phas[i][j];
		}
	}
}
